--Copyright ©2020, Wirepath Home Systems, LLC. All rights reserved.

epi_protocol = {}
epi_protocol._VERSION = "epi_protocol 1.0.0"
JSON=require "json"

-------------------------------------------------------------------------------------------
--							Hansong supplied command codes
-------------------------------------------------------------------------------------------

epi_commands = {}
--- system ---
-------------------------------------------------------------------------------------------
--							Main api section 
-------------------------------------------------------------------------------------------
--attrib: https://www.lua.org/pil/11.4.html
PacketList = {}
function PacketListNew()
	return {first = 0, last = -1}
end

function PacketList.new()
	return {first = 0, last = -1}
end
	
function PacketList.pushright(list, value)
	local last = list.last + 1
	list.last = last
	list[last] = value
end
	
function PacketList.popleft(list)
	local first = list.first
	if first > list.last then error("list is empty") end
	local value = list[first]
	list[first] = nil        -- to allow garbage collection
	list.first = first + 1
	return value
end
-- attrib end

function epi_protocol.init(outputCount, inputCount)
   if (epi_protocol.isInited == nil or epi_protocol.isInited == false) then
	   g_outputCount = outputCount
	   g_inputCount = inputCount
	   g_packetList = PacketListNew()
	   g_outstandingPacketCount = 0
	   g_outstadingPacket = false
	   epi_protocol.isInited = true
       g_pqwdTimer = nil
       g_wdCheck1 = false
       g_wdCheck2 = false
   end
end

-- create the json outgoing packet from inputs
function epi_protocol.createPacket(tableData)
	local packet = JSON:encode(tableData)
	return packet
end


function epi_protocol.sendPacket(hexData, noPadding)
	-- an optimization can be made here - if queue is empty, we don't need to queue
	if ((noPadding ~= nil) and (noPadding ~= false)) then
	  -- hexData = hexData .. "\n"
	  -- hexData = hexData .. tohex("00")
	end
	PacketList.pushright(g_packetList, hexData)
	epi_protocol.SendFromQueue()
end

function epi_protocol.responseReceived()
    if (g_connected == false) then
        C4:SetBindingStatus(6001, "online")
        C4:UpdateProperty("Connection Status", "online")
        g_connected = true
    end
	g_outstadingPacket = false
    g_wdCheck1 = false
    g_wdCheck2 = false
	epi_protocol.SendFromQueue()
end

function epi_protocol.SendFromQueue()
	if ( (g_outstadingPacket == false) and (g_packetList.first <= g_packetList.last)) then
		local hexData = PacketList.popleft(g_packetList)
		if (Log._logLevel > 3) then 
			printd(string.format("%.2f", socket.gettime() - g_startTime) .. ":sending packet-->")
			hexdump(hexData)
		end
		C4:SendToNetwork(6001, 8080, hexData)
		g_outstadingPacket = true
        return
    end
	if  (g_outstadingPacket == true) then
        printd("epi_protocol.SendFromQueue() called - waiting on outstanding packet")
    end
end

function epi_protocol.setPowerOnMethod(val)
	local data = {
		type = "set_automode",
		value = tostring(val)
	}
	packet = epi_protocol.createPacket(data)
	printd("Sending setPowerOnMethod packet of ".. packet)
	epi_protocol.sendPacket(packet)
end

function epi_protocol.setSleepMode(val)
	local data = {
		type = "set_sleepmode",
		value = tostring(val)
	}
	packet = epi_protocol.createPacket(data)
	printd("Sending setSleepMode packet of ".. packet)
	epi_protocol.sendPacket(packet)
end

function epi_protocol.login(userName, password)
	local data = {}
	data["type"] = "login"
	data["username"] = userName
	data["password"] = password
	packet = epi_protocol.createPacket(data)
	--hackPacket = '{"type":"login", "username":"episode", "password":"notsecure"}'
	printd("Sending login packet of ".. packet)
	epi_protocol.sendPacket(packet)
end

function epi_protocol.setAmpName(newName)
	local data = {}
	data["type"] = "set_ampname"
	data["value"] = newName
	packet = epi_protocol.createPacket(data)
	printd("Sending setAmpName packet of ".. packet)
	epi_protocol.sendPacket(packet)
end

function epi_protocol.setPowerOn()
	local data = {}
	data["type"] = "set_powerstatus"
	data["value"] = "1"
	packet = epi_protocol.createPacket(data)
	printd("Sending setPowerStatusOn packet of ".. packet)
	epi_protocol.sendPacket(packet)
end

function epi_protocol.setPowerOff()
	local data = {}
	data["type"] = "set_powerstatus"
	data["value"] = "0"
	packet = epi_protocol.createPacket(data)
	printd("Sending setPowerOff packet of ".. packet)
	epi_protocol.sendPacket(packet)
end

function epi_protocol.getPowerStatus()
	local data = {}
	data["type"] = "get_powerstatus"
	packet = epi_protocol.createPacket(data)
	printd("Sending getPowerStatus packet of ".. packet)
	epi_protocol.sendPacket(packet)
end

function PackQueueWatchdogTimerHandler(timer, skip)
    -- so this fires every 3 seconds to check for stuck queue.  If we have sent a packet w/o getting a resonse, set the g_wdCheck1 bit.
    -- clear it on every receiption
    -- if we get back into this time and the g_wdCheck1 bit is still set that maybe ok, we perhaps just sent out a packet, but set the g_wdCheck2 bit.
    -- if we get back into this and the g_wdCheck2 is set, we are stuck (over 3 seconds of nothing received.
    -- why not just set a timer when sending a packet and canel it when it is received?  Would work, but lots more os overhead then one reoccurring timer and two bit of state
    if (g_outstadingPacket == false) then return end
    if (g_wdCheck2 == true) then
        -- unjam and continue I guess for now
        print("ERROR - packet sending watch dog timer hit!!!!")
-- don't do this - they will not ever call our poll again and we will
-- never send another packet C4:SetBindingStatus(6001, "offline")
        C4:UpdateProperty("Connection Status", "offline")
        g_connected = false
        g_outstadingPacket = false
        g_wdCheck1 = false
        g_wdCheck2 = false
        epi_protocol.SendFromQueue()
    end
    if (g_wdCheck1 == true) then
        g_wdCheck2 = true
    else
        g_wdCheck1 = true
    end
end

function startPacketQueueWatchdogTimer()
   if (g_pqwdTimer ~= nil) then g_pqwdTimer:Cancel() end
   g_pqwdTimer = C4:SetTimer(3000, PackQueueWatchdogTimerHandler, true)
   g_wdCheck1 = false
   g_wdCheck2 = false
end

function epi_protocol.ConnectionStatusChanged(connected)
    if (connected) then
        g_outstadingPacket = false
        g_wdCheck1 = false
        g_wdCheck2 = false
	    epi_protocol.SendFromQueue()
        startPacketQueueWatchdogTimer()
    else
        if (g_pqwdTimer ~= nil) then g_pqwdTimer:Cancel() end
    end
end

function epi_protocol.setCECOn()
	local data = {
		type = "set_cecsetting",
		value = {
		   {
			cecEnable = "1",
            volumeEnable = "1"
		   }
		}
	}
	packet = epi_protocol.createPacket(data)
	printd("Sending setCECOn packet of ".. packet)
	epi_protocol.sendPacket(packet)
end

function epi_protocol.setCECOff()
	local data = {
		type = "set_cecsetting",
		value = {
		   {
			cecEnable = "0",
            volumeEnable = "0"
		   }
		}
	}
	packet = epi_protocol.createPacket(data)
	printd("Sending setCECOff packet of ".. packet)
	epi_protocol.sendPacket(packet)
end

function epi_protocol.setVolume(outputIndex, volume)
--	local vol = convert100BasedVolTo80dbBasedVol(volume)
	local vol = tostring(volume)
	local data = {
		type = "set_mastervolume",
		value = {
		   {
			index = outputIndex - 1,
			masterVolume = vol,
                     	mute = "0"
		   }
		}
	}
	packet = epi_protocol.createPacket(data)
	printd("Sending setVolume packet of ".. packet)
	epi_protocol.sendPacket(packet)
end

function epi_protocol.getVolumes()
   local data = { type = "get_mastervolume" }
	packet = epi_protocol.createPacket(data)
	printd("Sending getVolumes packet of ".. packet)
	epi_protocol.sendPacket(packet)
end

function epi_protocol.setTurnOnVolume(outputIndex, volume)
-- no way to set the power on volume??
end

function epi_protocol.setOutputMuteState(outputIndex, muted)
	local data = {
		type = "set_mastervolume",
		value = {
		   {
			index = tonumber(outputIndex) - 1,
                     	mute = tostring(muted)
		   }
		}
	}
	packet = epi_protocol.createPacket(data)
	printd("Sending setVolume packet of ".. packet)
	epi_protocol.sendPacket(packet)
end

function epi_protocol.setOutputToInput(outputIndex, inputIndex)
	local data = {
		type = "set_inputsetting",
		value = {
		   {
			index = tonumber(inputIndex) - 1,
			inputEnable = "1",
		   }
		}
	}
	packet = epi_protocol.createPacket(data)
	printd("Sending setOutputToInput packet of ".. packet)
	epi_protocol.sendPacket(packet)
end

function epi_protocol.disconnectOutput(outputIndex)
	-- don't have a disconnect yet - just mute for now I guess
end


function epi_protocol.getInputSetting(inputId)
	local data = {
		type = "get_inputsetting",
        value = {
			index = tonumber(inputId) - 1
        }
	}
	packet = epi_protocol.createPacket(data)
	printd("Sending getInputSetting packet of ".. packet)
	epi_protocol.sendPacket(packet)
end

function epi_protocol.setSurroundMode(inputId, soundMode)
	local data = {
		type = "set_inputsetting",
		value = {
		   {
			index = tonumber(inputId) - 1,
			soundMode = tostring(soundMode)
		   }
		}
	}
	packet = epi_protocol.createPacket(data)
	printd("Sending setSurroundMode packet of ".. packet)
	epi_protocol.sendPacket(packet)
end

function epi_protocol.getOutputSetting()
	local data = {
		type = "get_outputsetting"
	}
	packet = epi_protocol.createPacket(data)
	printd("Sending getOutputSetting packet of ".. packet)
	epi_protocol.sendPacket(packet)
end

function epi_protocol.setIpAddress(dhcp, ipaddress, gatewayaddress, dns, maskaddress)
	local data = {
		type = "set_ip",
		value = {
		   {
				dhcp = dhcp,
				ipaddress = ipaddress,
				gateway = gatewayaddress,
				subnetmask = maskaddress,
				dnsserver = dns
		   }
		}
	}
	packet = epi_protocol.createPacket(data)
	printd("Sending setIpAddress packet of ".. packet)
	epi_protocol.sendPacket(packet)
end

function epi_protocol.getIpAddress()
	local data = {
		type = "get_ip"
	}
	packet = epi_protocol.createPacket(data)
	printd("Sending getIpAddress packet of ".. packet)
	epi_protocol.sendPacket(packet)
end

function epi_protocol.getGeneralSetting()
	local data = {
		type = "get_generalsetting"
	}
	packet = epi_protocol.createPacket(data)
	printd("Sending getGeneralSetting packet of ".. packet)
	epi_protocol.sendPacket(packet)
end

function epi_protocol.setGeneralSetting(standbyState0or1, hdmiAudioEnableOnofOff)
	local data = {
		type = "set_generalsetting",
		value = {
		   {
			 standbyEnable = standbyState0or1,
			 hdmiAudioEnable = hdmiAudioEnableOnofOff
		   }
		}
	}
	packet = epi_protocol.createPacket(data)
	printd("Sending setGeneralSetting packet of ".. packet)
	epi_protocol.sendPacket(packet)
end

function epi_protocol.getCecSetting()
	local data = {
		type = "get_cecsetting"
	}
	packet = epi_protocol.createPacket(data)
	printd("Sending getIpAddress packet of ".. packet)
	epi_protocol.sendPacket(packet)
end

function epi_protocol.getSystemSetting()
	local data = {
		type = "get_systemsetting"
	}
	packet = epi_protocol.createPacket(data)
	printd("Sending getSystemSetting packet of ".. packet)
	epi_protocol.sendPacket(packet)
end

-------------------------------------------------------------------------------------------
--							SOME HELPER FUNCTIONS
-------------------------------------------------------------------------------------------

function ToInteger(number)
	return math.floor(tonumber(number) or error("Could not cast '" .. tostring(number) .. "' to number.'"))
end


function hexdump(buf, printFunc)
	local outtable, ins = {}, table.insert
	printFunc = printFunc or print

	for byte=1, #buf, 16 do
		local chunk = buf:sub(byte, byte+15)
		ins(outtable, string.format('%08X  ',byte-1))

		local chunkleft, chunkright = chunk:sub(1, 8), chunk:sub(9)
		chunkleft:gsub('.', function (c) ins(outtable, string.format('%02X ',string.byte(c))) end)
		ins(outtable, ' ')

		chunkright:gsub('.', function (c) ins(outtable, string.format('%02X ',string.byte(c))) end)
		ins(outtable, string.rep(' ',3*(16-#chunk)))

		chunk = chunk:gsub("[^%p%w%s]", '.')
		chunk = chunk:gsub("[\r\n]", '.')
		ins(outtable, ' ' .. chunk .. '\r\n')
	end

	printFunc(table.concat(outtable))
end

function tohex(str)
	local offset,dif = string.byte("0"), string.byte("A") - string.byte("9") - 1
	local hex = ""
	str = str:upper()
	for a,b in str:gfind "(%S)(%S)%s*" do
		a,b = a:upper():byte() - offset, b:upper():byte()-offset
		a,b = a>10 and a - dif or a, b>10 and b - dif or b
		local code = a*16+b
		hex = hex .. string.char(code)
	end
	return hex
end


