--------------------------------------------------------------------------------------------
--	AlphaMap.lua
--
--  Copyright 2014 Control4 Corporation. All rights reserved.
--
--  Takes an array and generates an alpha map
--------------------------------------------------------------------------------------------

require "law.objects"
require "law.StringBuilder"
require "law.Utf8"
require "law.Xml"

AlphaMap = inheritsFrom(nil)

--[[
	Creates an alphaMap.
	@param nameProperty the property of the object that holds the name that should be indexed
	@param arrayOfObjs An array of objects (tables) that is what is being indexed.
--]]
function AlphaMap:init(nameProperty, arrayOfObjs)
	self._arrayOfObjs = arrayOfObjs
	self._nameProperty = nameProperty
	self._xml = nil
end

--[[
	Converts the alpha map to MSP xml
--]]
function AlphaMap:toXml()
	if self._xml then
		return self._xml
	end

	local list = self:toList()
	local sb = StringBuilder:new()
	local currentChar = ""

	sb:add("<AlphaMap>")

	for item in self:iter() do
		if (item.index == 0 and string.match (item.key, '%A')) then
			item.key = '#'
		end

		if (string.match (item.key, '^[%u%#]$')) then
			sb:add("<item>")
			sb:add("<key>")
			sb:add(C4:XmlEscapeString(item.key))
			sb:add("</key>")

			sb:add("<index>")
			sb:add(item.index) -- 0 based
			sb:add("</index>")
			sb:add("</item>")
		end
	end

	sb:add("</AlphaMap>")

	self._xml = tostring(sb)

	return self._xml
end

function AlphaMap:toList()
	local list = {}
	for item in self:iter() do
		list[#list + 1] = item
	end
	return list
end


--[[
	Returns an item iterable
--]]
function AlphaMap:iter()
	local index = 1
	local currentChar = nil
	return function()
		while index <= #self._arrayOfObjs do
			local nameStr = self._arrayOfObjs[index][self._nameProperty]
			index = index + 1
			if nameStr then
				local trimmed = nameStr:lTrim()

				local str = Utf8:new(trimmed)
				local char = str:charAt(1)

				if (char ~= currentChar and
					(char:len() == 1 and
						char:upper() ~= currentChar and
						char:lower() ~= currentChar
					)
				)
				then
					currentChar = char
					item = {}
					item.key = char:upper ()
					item.index = index - 2
					return item
				end
			end
		end
		return nil
	end
end