--------------------------------------------------------------------------------------------
--  CsvFileReader.lua
--
--  Copyright 2015 Control4 Corporation. All rights reserved.
--  
--  A simple reader of csv file format. Note this class doesn't currently handle embedded
--  line breaks, commas, or double quotes.
-- 
--------------------------------------------------------------------------------------------

require "law.objects"
require "law.StringUtils"
require "law.Log"

CsvFileReader = inheritsFrom(nil)

function CsvFileReader:init(filePath)
	local status, f = pcall(io.open, filePath, "r")

	if not status or not f then
		Log:Warn("Could not open file:", filePath)
		return
	end

	-- read the file
	status, self._data = pcall(f.read, f, "*a")

	if not status then 
		Log:Warn("Could not read file:", filePath)		
	end

	status = pcall(f.close, f)

	if not status then 
		Log:Warn("Could not close file:", filePath)
	end
end

--[[
	The length of the file in bytes.
--]]
function CsvFileReader:len()
	if not self._data then
		return 0
	end
	
	return self._data:len()
end

--[[
	Iterates each line returning the cols.
--]]
function CsvFileReader:line()
	if not self._data or self._data:len() == 0 then
		-- Nothing to do here.
		return function() return nil end
	end

	local index = 1 -- where to start search

	return function()
		local line
		line, index = self._data:match("(.-)\r*\n()", index)
		if line and line:len() > 0 then 
			local rows = line:split("%s*,%s*")			
			return unpack(rows)
		end

		return nil
	end
end