--------------------------------------------------------------------------------------------
--	LinkedList.lua
--  
--  Copyright 2014 Control4 Corporation. All rights reserved.
--
--  Implements a linked list in lua
--------------------------------------------------------------------------------------------

require "law.objects"

LinkedList = inheritsFrom(nil)

function LinkedList:init()
	-- pointers
	self._start = nil
	self._end = nil
	self._length = 0
end

function LinkedList:length()
	return self._length
end

function LinkedList:add(item)
	local linkItem = 
	{
		val = item
	}

	if self._end then
		local last = self._end
		last._next = linkItem
		linkItem._prev = last
		self._end = linkItem
	else
		self._start = linkItem
		self._end = linkItem
	end

	self._length = self._length + 1
	return linkItem
end

function LinkedList:remove(linkListItem)
	if not linkListItem then 
		return
	end

	local prevItem = linkListItem._prev
	local nextItem = linkListItem._next

	if prevItem and nextItem then
		prevItem._next = nextItem
		nextItem._prev = prevItem
	elseif nextItem then
		-- Start node
		self._start = nextItem
		nextItem._prev = nil
	elseif prevItem then
		-- End node
		self._end = prevItem
		prevItem._next = nil
	else
		-- Only node
		self._start = nil
		self._end = nil
	end

	self._length = self._length - 1
end

--[[
	Iterates over all elements
--]]
function LinkedList:all()
	local curNode = nil
	return function()
		if not curNode then
			curNode = self._start
			return self._start
		end
		local next = curNode._next
		curNode = curNode._next
		return next
	end
end

--[[
	Iterates over all elements in reverse
--]]
function LinkedList:allReverse()
	local curNode = nil
	return function()
		if not curNode then
			curNode = self._end
			return self._end
		end
		local prev = curNode._prev
		curNode = curNode._prev
		return prev
	end
end
