--------------------------------------------------------------------------------------------
--  Copyright 2014 Control4 Corporation. All rights reserved.
--------------------------------------------------------------------------------------------

--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-- Debug Logging Code
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
Logger = {}

-- Create a Table with Logging functions
function Logger:Create()
  
  -- table for logging functions
  local lt = {}
  
  lt._logLevel = 0
  lt._outputPrint = false
  lt._outputC4Log = false
  lt._logName =  __LOG_NAME__ or "Set Log Name"
  lt._index = 1
  
  function lt:SetLogLevel(level)
    self._logLevel = level
  end
  
  function lt:OutputPrint(value)
    self._outputPrint = value
  end
  
  function lt:OutputC4Log(value)
    self._outputC4Log = value
  end
  
  function lt:SetLogName(name)
    self._logName = name
  end

  function lt:Enabled()
    return (self._outputPrint or self._outputC4Log)
  end
  
  function lt:PrintTable(tValue, sIndent)
    if not sIndent then
      sIndent = "" 
    end

    if (type(tValue) == "table") then
      if (self._outputPrint) then
        for k,v in pairs(tValue) do
          print(self:_createLogHeader() .. sIndent .. tostring(k)  .. ":  " .. (tostring(v) or ""))
          if (type(v) == "table") then
            self:PrintTable(v, sIndent .. "   ")
          end
        end
      end
  
      if (self._outputC4Log) then
        for k,v in pairs(tValue) do
          C4:ErrorLog(self:_createLogHeader() .. sIndent .. tostring(k) .. ":  " .. tostring(v))
          if (type(v) == "table") then
            self:PrintTable(v, sIndent .. "   ")
          end
        end
      end

    else
      if (self._outputPrint) then
        print (self:_createLogHeader() .. sIndent .. tValue)
      end
      
      if (self._outputC4Log) then
        C4:ErrorLog(self:_createLogHeader() .. sIndent .. tValue)
      end
    end
  end
  
  function lt:getNextIndex()
    local i = self._index
    self._index = self._index + 1
    return string.format(" %04d ", i)
  end

  function lt:_createLogHeader()
    return self._logName .. ": [" .. os.date("%x %X") .. self:getNextIndex() .. "] ::"
  end

  function lt:Print(logLevel, ...)
    local args = {...}

    if (self._logLevel >= logLevel) then
      for i,v in ipairs(args) do
        if (type(v) == "table") then
          self:PrintTable(v, "   ")
          return
          else
        end
      end
      
      if (self._outputPrint) then
        print (self:_createLogHeader(), ...)
      end

      if (self._outputC4Log) then
        local msg = ""
        for i,v in ipairs(args) do
          local space = " "
          if i == 1 then 
            space = ""
          end
          msg = msg .. space .. (tostring(v) or "nil")
        end
        C4:ErrorLog(self:_createLogHeader() .. msg )
      end
    end
  end
  
  function lt:Alert(...)
    self:Print(0, "Alert:", ...)
  end
  
  function lt:Error(...)
    self:Print(1, "Err:", ...)
  end
  
  function lt:Warn(...)
    self:Print(2, "Warn:", ...)
  end
  
  function lt:Info(...)
    self:Print(3, "Info:", ...)
  end
  
  function lt:Debug(...)
    self:Print(4, "Dbg:", ...)
  end
  
function lt:Trace(...)
    self:Print(5, "Trace:", ...)
  end

  return lt
end

-- Global log 
if not Log then     
  Log = Logger.Create()
end


