--------------------------------------------------------------------------------------------
--	TimerManager.lua
--  
--  Copyright 2014 Control4 Corporation. All rights reserved.
--------------------------------------------------------------------------------------------
require("law.ServiceLocator")
require("law.TimerPromise")

--[[
	Handles C4 Timer expiration
--]]
function OnTimerExpired(timerId)
	ServiceLocator.TimerManager:onTimerExpired(timerId)
end

TimerManager = inheritsFrom(nil)

function TimerManager:init(addToServiceLocator)
	self._timerHandles = {}

	if addToServiceLocator then
		ServiceLocator:addService("TimerManager", self)
	end
end

--[[
	Adds a timer to the timer manager
	@param milliseconds Number of milliseconds between timer firing
	@return Promise a promise that the timer will fire in the future.
--]]
function TimerManager:addTimer(milliseconds)
	local timerId = C4:AddTimer(milliseconds, "MILLISECONDS", false)
	local promise = TimerPromise:new(timerId)

	self._timerHandles[timerId] = promise
	return promise
end

--[[
	Kills a timer with the given timer id.
	Calls reject on the associated promise
--]]
function TimerManager:killTimer(timerId)
	C4:KillTimer(timerId)
	local promise = self._timerHandles[timerId]
	self._timerHandles[timerId] = nil

	if promise then
		promise:reject("Timer Killed")
	end
end

--[[
	Clears all timers
--]]
function TimerManager:destroy()
	for i,promise in pairs(self._timerHandles) do
		promise:reject("Timer Killed")
	end
	self._timerHandles = {}
end

function TimerManager:onTimerExpired(timerId)
	local promise = self._timerHandles[timerId]
	if promise then
		promise:resolve()
	end

	self._timerHandles[timerId] = nil
end
